package viewModel

import kotlinx.coroutines.flow.MutableStateFlow
import kotlinx.coroutines.flow.StateFlow
import kotlinx.serialization.SerializationException
import kotlinx.serialization.encodeToString
import kotlinx.serialization.json.Json
import java.io.*

object ShortCutHandler {
    private val currentPath: File =
        File(System.getProperty("user.home"), ".QuickLaunch")
    private val shortcutsFile = File(currentPath,"shortcuts.json")
    private val _shortcuts = MutableStateFlow<Map<String, String>>(emptyMap())
    val shortcuts: StateFlow<Map<String, String>> = _shortcuts


    init {
        if (!currentPath.exists()){ //TODO this is for config somewhere else maybe
            currentPath.mkdir()
        }
        readFromStorage()
//        BufferedReader(FileReader(File("C:\\Users\\Paul\\AppData\\Roaming\\QuickLaunch\\File.txt"))).use {
//            _shortcuts.value = buildMap {
//                var line = it.readLine()
//                while (line != null){
//                    put(line, it.readLine())
//                    line = it.readLine()
//                }
//            }
//        }
//        persistToStorage()
    }

    fun addShortcut(short: String, path: String) {
        _shortcuts.value = buildMap {
            putAll(shortcuts.value)
            put(short, path)
        }
    }

    fun removeShortcut(short: String) {
        _shortcuts.value = buildMap {
            putAll(shortcuts.value.filterNot { entry -> entry.key == short })
        }
    }

    fun executeShortcut(short: String) {
        shortcuts.value[short]?.let {
            ProcessBuilder(it.split(";")).start()
        }
    }

    fun persistToStorage() {
        _shortcuts.value = buildMap {
            putAll(shortcuts.value.filter { entry -> entry.key.isNotEmpty() })
        }
        BufferedWriter(FileWriter(shortcutsFile)).use {
            it.write(
                Json.Default.encodeToString(shortcuts.value)
            )
        }
    }

    private fun readFromStorage() {
        println(shortcutsFile.path)
        if (!shortcutsFile.exists()) {
            shortcutsFile.createNewFile()
        } else {
            try {
                BufferedReader(FileReader(shortcutsFile)).use {
                    _shortcuts.value = Json.Default.decodeFromString(it.readText())
                }
            } catch (j : SerializationException){
                println("Empty configuration")
            }
        }

    }
}
